package com.gd.armageddon
{
	import fl.text.TLFTextField;
	
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.media.Sound;
	import flash.utils.Dictionary;
	import flash.media.SoundChannel;
	
	public dynamic class Armageddon extends Sprite
	{
		private static var _game:Armageddon;
		
		public static function get armageddon():Armageddon
		{
			return _game;
		}
		
		[Inspectable(defaultValue="world")]
		public var firstLocale:String = "world";
		
		public var counter:TLFTextField;
		public var clock:Clock;
		public var logo:Logo;
		public var bgButton:Clicker;
		
		private var locales:Dictionary;
		private var stack:Array;
		private var lastTime:int;
		
		private var pauseSound:SoundChannel;
		
		private var state:uint;
		
		private static const STATE_PLAYING:uint = 0;
		private static const STATE_GAME_OVER:uint = 1;
		
		public function Armageddon():void
		{
			// listen for initial load to insure settings are active to set into global vars
			loaderInfo.addEventListener(Event.COMPLETE, init);
		}
		private function init(e:Event):void
		{
			// remove the listener
			loaderInfo.removeEventListener(Event.COMPLETE, init);
			
			stage.addEventListener(MouseEvent.CLICK, clickedStage, false, 0, true);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, pressedKey, false, 0, true);
			
			counter.mouseEnabled = false;
			counter.mouseChildren = false;
			
			clock = new Clock();
			clock.addEventListener(Event.CHANGE, timerChanged, false, 0, true);
			clock.running = true;
			
			_game = this;
			
			bgButton.onClick = function():void
			{
				if(stack.length > 1)
				{
					popInstance();
					clock.forceRunFromCurrentTimeTo(scaleTimeCost(1));
				}
			}
			
			logo.button.onClick = function():void
			{
				beginGame();
			}
			
			stack = new Array();
			
			locales = new Dictionary();
			for (var i:int = this.numChildren - 1; i >= 0; i--)
			{
				var c:* = this.getChildAt(i);
				if(c is Locale)
				{
					addLocales(c);
				}
			}
			
			beginGame();
		}
		
		protected function pressedKey(event:KeyboardEvent):void
		{
			if(state == STATE_PLAYING && event.keyCode == 32)
			{
				var shouldRun:Boolean = !clock.running;
				
				if(pauseSound)
				{
					pauseSound.stop();
					pauseSound = null;
				}
				
				var sound:Sound = shouldRun ? new Unpause() : new Pause();
				pauseSound = sound.play();
				
				clock.running = !clock.running;
			}
		}
		
		protected function clickedStage(event:MouseEvent):void
		{
//			if(stack.length > 1)
//			{
//				this.popInstance();
//				clock.forceRunFromCurrentTimeTo(scaleTimeCost(1));
//			}
		}
		
		private function addLocales(... locales):void
		{
			for each(var l:Locale in locales)
			{
				this.locales[l.name] = l;
			}
		}
		
		private function beginGame():void
		{
			state = STATE_PLAYING;
			logo.visible = false;
			bgButton.visible = true;
			clock.reset();
			clock.running = true;
			lastTime = Math.ceil(clock.timeRemaining);
			
			for each(var l:Locale in locales)
			{
				l.reset();
			}
			
			counter.visible = true;
			this.tryPushingInstance(this.firstLocale);
		}
		
		private function gameOver():void
		{
			state = STATE_GAME_OVER;
			
			clearAllInstances();
			
			logo.visible = true;
			bgButton.visible = false;
			logo.gotoAndPlay(0);
			
			counter.visible = false;
		}
		
		public function payTimeCost(timeCost:Number = 0):void
		{
			clock.forceRunFromCurrentTimeTo(scaleTimeCost(timeCost));
		}
		
		public function activatePortal(portal:Portal):void
		{
			var locale:Locale = locales[portal.toInstance];
			if(locale)
			{
				this.tryPushingInstance(portal.toInstance);
				
				clock.forceRunFromCurrentTimeTo(scaleTimeCost(1 + portal.timeCost));
			}
		}
		
		public static function scaleTimeCost(cost:Number):int
		{
			return Math.ceil(cost * 1000 / 3);
		}
		
		public function tryPushingInstance(name:String):void
		{
			var locale:Locale = locales[name];
			if(locale)
			{
				stack.push(locale);
				updateStack();
			}
			
			trace("Try push locale: " + name);
		}
		
		public function popInstance():void
		{
			stack.pop();
			updateStack();
		}
		
		private function clearAllInstances():void
		{
			stack = [];
			updateStack();
		}
		
		private function updateStack():void
		{
			var locale:Locale = null;
			if(stack.length > 0) locale = stack[stack.length - 1];
			
			for each(var l:Locale in locales)
			{
				l.visible = false;
			}
			
			if(locale)
			{
				locale.visible = true;
			}
			
			bgButton.alpha = Math.min(1, stack.length / 12); 
		}
		
		protected function timerChanged(event:Event):void
		{
			var remaining:Number = clock.timeRemaining;
			
			if(remaining < lastTime && lastTime > 0)
			{
				new Beep().play();
				
				lastTime--;
			}
			
			var timeRemaining:String = remaining.toFixed(2);
			var beforeAfter:Array = timeRemaining.split(".");
			
			if(beforeAfter[0].length < 2)
			{
				timeRemaining = "0" + timeRemaining;
			}
			
			if(beforeAfter[1].length < 2)
			{
				timeRemaining = timeRemaining + "0";
			}
			
			counter.text = timeRemaining;
			
			if(clock.timeRemaining == 0)
			{
				this.gameOver();
			}
			
			for each(var l:Locale in locales)
			{
				l.update();
			}
		}
	}
}